package jdk.internal.org.objectweb.asm.tree;

import jdk.internal.org.objectweb.asm.ClassVisitor;
import jdk.internal.org.objectweb.asm.Label;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Type;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;
import java.util.Arrays;
import java.util.ArrayList;
import jdk.internal.org.objectweb.asm.Attribute;
import java.util.List;
import jdk.internal.org.objectweb.asm.MethodVisitor;

public class MethodNode extends MethodVisitor
{
    public int access;
    public String name;
    public String desc;
    public String signature;
    public List<String> exceptions;
    public List<ParameterNode> parameters;
    public List<AnnotationNode> visibleAnnotations;
    public List<AnnotationNode> invisibleAnnotations;
    public List<TypeAnnotationNode> visibleTypeAnnotations;
    public List<TypeAnnotationNode> invisibleTypeAnnotations;
    public List<Attribute> attrs;
    public Object annotationDefault;
    public List<AnnotationNode>[] visibleParameterAnnotations;
    public List<AnnotationNode>[] invisibleParameterAnnotations;
    public InsnList instructions;
    public List<TryCatchBlockNode> tryCatchBlocks;
    public int maxStack;
    public int maxLocals;
    public List<LocalVariableNode> localVariables;
    public List<LocalVariableAnnotationNode> visibleLocalVariableAnnotations;
    public List<LocalVariableAnnotationNode> invisibleLocalVariableAnnotations;
    private boolean visited;
    
    public MethodNode() {
        this(327680);
        if (this.getClass() != MethodNode.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    public MethodNode(int api) {
        super(api);
        this.instructions = new InsnList();
    }
    
    public MethodNode(int n, String s, String s2, String s3, String[] array) {
        this(327680, n, s, s2, s3, array);
        if (this.getClass() != MethodNode.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    public MethodNode(int api, int access, String name, String desc, String signature, String[] a) {
        super(api);
        this.access = access;
        this.name = name;
        this.desc = desc;
        this.signature = signature;
        this.exceptions = new ArrayList((a == null) ? 0 : a.length);
        if ((access & 0x400) == 0x0) {
            this.localVariables = new ArrayList(5);
        }
        this.tryCatchBlocks = new ArrayList();
        if (a != null) {
            this.exceptions.addAll(Arrays.asList(a));
        }
        this.instructions = new InsnList();
    }
    
    @Override
    public void visitParameter(String s, int n) {
        if (this.parameters == null) {
            this.parameters = new ArrayList(5);
        }
        this.parameters.add(new ParameterNode(s, n));
    }
    
    @Override
    public AnnotationVisitor visitAnnotationDefault() {
        return new AnnotationNode(new ArrayList<Object>(0) {
            @Override
            public boolean add(Object o) {
                MethodNode.this.annotationDefault = o;
                return super.add(o);
            }
        });
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String s, boolean b) {
        AnnotationNode annotationNode = new AnnotationNode(s);
        if (b) {
            if (this.visibleAnnotations == null) {
                this.visibleAnnotations = new ArrayList(1);
            }
            this.visibleAnnotations.add(annotationNode);
            return annotationNode;
        }
        else {
            if (this.invisibleAnnotations == null) {
                this.invisibleAnnotations = new ArrayList(1);
            }
            this.invisibleAnnotations.add(annotationNode);
            return annotationNode;
        }
    }
    
    @Override
    public AnnotationVisitor visitTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        TypeAnnotationNode typeAnnotationNode = new TypeAnnotationNode(n, typePath, s);
        if (b) {
            if (this.visibleTypeAnnotations == null) {
                this.visibleTypeAnnotations = new ArrayList(1);
            }
            this.visibleTypeAnnotations.add(typeAnnotationNode);
            return typeAnnotationNode;
        }
        else {
            if (this.invisibleTypeAnnotations == null) {
                this.invisibleTypeAnnotations = new ArrayList(1);
            }
            this.invisibleTypeAnnotations.add(typeAnnotationNode);
            return typeAnnotationNode;
        }
    }
    
    @Override
    public AnnotationVisitor visitParameterAnnotation(int n, String s, boolean b) {
        AnnotationNode annotationNode = new AnnotationNode(s);
        if (b) {
            if (this.visibleParameterAnnotations == null) {
                this.visibleParameterAnnotations = (List[])new List[Type.getArgumentTypes(this.desc).length];
            }
            if (this.visibleParameterAnnotations[n] == null) {
                this.visibleParameterAnnotations[n] = new ArrayList(1);
            }
            this.visibleParameterAnnotations[n].add(annotationNode);
            return annotationNode;
        }
        else {
            if (this.invisibleParameterAnnotations == null) {
                this.invisibleParameterAnnotations = (List[])new List[Type.getArgumentTypes(this.desc).length];
            }
            if (this.invisibleParameterAnnotations[n] == null) {
                this.invisibleParameterAnnotations[n] = new ArrayList(1);
            }
            this.invisibleParameterAnnotations[n].add(annotationNode);
            return annotationNode;
        }
    }
    
    @Override
    public void visitAttribute(Attribute attribute) {
        if (this.attrs == null) {
            this.attrs = new ArrayList(1);
        }
        this.attrs.add(attribute);
    }
    
    @Override
    public void visitCode() {
    }
    
    @Override
    public void visitFrame(int n, int n2, Object[] array, int n3, Object[] array2) {
        this.instructions.add(new FrameNode(n, n2, (array == null) ? null : this.getLabelNodes(array), n3, (array2 == null) ? null : this.getLabelNodes(array2)));
    }
    
    @Override
    public void visitInsn(int n) {
        this.instructions.add(new InsnNode(n));
    }
    
    @Override
    public void visitIntInsn(int n, int n2) {
        this.instructions.add(new IntInsnNode(n, n2));
    }
    
    @Override
    public void visitVarInsn(int n, int n2) {
        this.instructions.add(new VarInsnNode(n, n2));
    }
    
    @Override
    public void visitTypeInsn(int n, String s) {
        this.instructions.add(new TypeInsnNode(n, s));
    }
    
    @Override
    public void visitFieldInsn(int n, String s, String s2, String s3) {
        this.instructions.add(new FieldInsnNode(n, s, s2, s3));
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor);
            return;
        }
        this.instructions.add(new MethodInsnNode(opcode, owner, name, descriptor));
        return;
    }
    
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor, isInterface);
            return;
        }
        this.instructions.add(new MethodInsnNode(opcode, owner, name, descriptor, isInterface));
        return;
    }
    
    @Override
    public void visitInvokeDynamicInsn(String s, String s2, Handle handle, Object... array) {
        this.instructions.add(new InvokeDynamicInsnNode(s, s2, handle, array));
    }
    
    @Override
    public void visitJumpInsn(int n, Label label) {
        this.instructions.add(new JumpInsnNode(n, this.getLabelNode(label)));
    }
    
    @Override
    public void visitLabel(Label label) {
        this.instructions.add(this.getLabelNode(label));
    }
    
    @Override
    public void visitLdcInsn(Object o) {
        this.instructions.add(new LdcInsnNode(o));
    }
    
    @Override
    public void visitIincInsn(int n, int n2) {
        this.instructions.add(new IincInsnNode(n, n2));
    }
    
    @Override
    public void visitTableSwitchInsn(int n, int n2, Label label, Label... array) {
        this.instructions.add(new TableSwitchInsnNode(n, n2, this.getLabelNode(label), this.getLabelNodes(array)));
    }
    
    @Override
    public void visitLookupSwitchInsn(Label label, int[] array, Label[] array2) {
        this.instructions.add(new LookupSwitchInsnNode(this.getLabelNode(label), array, this.getLabelNodes(array2)));
    }
    
    @Override
    public void visitMultiANewArrayInsn(String s, int n) {
        this.instructions.add(new MultiANewArrayInsnNode(s, n));
    }
    
    @Override
    public AnnotationVisitor visitInsnAnnotation(int n, TypePath typePath, String s, boolean b) {
        AbstractInsnNode abstractInsnNode;
        for (abstractInsnNode = this.instructions.getLast(); abstractInsnNode.getOpcode() == -1; abstractInsnNode = abstractInsnNode.getPrevious()) {}
        TypeAnnotationNode o = new TypeAnnotationNode(n, typePath, s);
        if (b) {
            if (abstractInsnNode.visibleTypeAnnotations == null) {
                abstractInsnNode.visibleTypeAnnotations = new ArrayList(1);
            }
            abstractInsnNode.visibleTypeAnnotations.add(o);
            return o;
        }
        else {
            if (abstractInsnNode.invisibleTypeAnnotations == null) {
                abstractInsnNode.invisibleTypeAnnotations = new ArrayList(1);
            }
            abstractInsnNode.invisibleTypeAnnotations.add(o);
            return o;
        }
    }
    
    @Override
    public void visitTryCatchBlock(Label label, Label label2, Label label3, String s) {
        this.tryCatchBlocks.add(new TryCatchBlockNode(this.getLabelNode(label), this.getLabelNode(label2), this.getLabelNode(label3), s));
    }
    
    @Override
    public AnnotationVisitor visitTryCatchAnnotation(int n, TypePath typePath, String s, boolean b) {
        TryCatchBlockNode tryCatchBlockNode = (TryCatchBlockNode)this.tryCatchBlocks.get((n & 0xFFFF00) >> 8);
        TypeAnnotationNode typeAnnotationNode = new TypeAnnotationNode(n, typePath, s);
        if (b) {
            if (tryCatchBlockNode.visibleTypeAnnotations == null) {
                tryCatchBlockNode.visibleTypeAnnotations = new ArrayList(1);
            }
            tryCatchBlockNode.visibleTypeAnnotations.add(typeAnnotationNode);
            return typeAnnotationNode;
        }
        else {
            if (tryCatchBlockNode.invisibleTypeAnnotations == null) {
                tryCatchBlockNode.invisibleTypeAnnotations = new ArrayList(1);
            }
            tryCatchBlockNode.invisibleTypeAnnotations.add(typeAnnotationNode);
            return typeAnnotationNode;
        }
    }
    
    @Override
    public void visitLocalVariable(String s, String s2, String s3, Label label, Label label2, int n) {
        this.localVariables.add(new LocalVariableNode(s, s2, s3, this.getLabelNode(label), this.getLabelNode(label2), n));
    }
    
    @Override
    public AnnotationVisitor visitLocalVariableAnnotation(int n, TypePath typePath, Label[] array, Label[] array2, int[] array3, String s, boolean b) {
        LocalVariableAnnotationNode localVariableAnnotationNode = new LocalVariableAnnotationNode(n, typePath, this.getLabelNodes(array), this.getLabelNodes(array2), array3, s);
        if (b) {
            if (this.visibleLocalVariableAnnotations == null) {
                this.visibleLocalVariableAnnotations = new ArrayList(1);
            }
            this.visibleLocalVariableAnnotations.add(localVariableAnnotationNode);
            return localVariableAnnotationNode;
        }
        else {
            if (this.invisibleLocalVariableAnnotations == null) {
                this.invisibleLocalVariableAnnotations = new ArrayList(1);
            }
            this.invisibleLocalVariableAnnotations.add(localVariableAnnotationNode);
            return localVariableAnnotationNode;
        }
    }
    
    @Override
    public void visitLineNumber(int n, Label label) {
        this.instructions.add(new LineNumberNode(n, this.getLabelNode(label)));
    }
    
    @Override
    public void visitMaxs(int maxStack, int maxLocals) {
        this.maxStack = maxStack;
        this.maxLocals = maxLocals;
    }
    
    @Override
    public void visitEnd() {
    }
    
    protected LabelNode getLabelNode(Label label) {
        if (!(label.info instanceof LabelNode)) {
            label.info = new LabelNode();
        }
        return (LabelNode)label.info;
    }
    
    private LabelNode[] getLabelNodes(Label[] array) {
        LabelNode[] array2 = new LabelNode[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = this.getLabelNode(array[i]);
        }
        return array2;
    }
    
    private Object[] getLabelNodes(Object[] array) {
        Object[] array2 = new Object[array.length];
        Object labelNode;
        for (int i = 0; i < array.length; array2[i] = labelNode, ++i) {
            labelNode = array[i];
            if ((labelNode instanceof Label)) {
                labelNode = this.getLabelNode((Label)labelNode);
            }
        }
        return array2;
    }
    
    public void check(int n) {
        if (n == 262144) {
            if (this.visibleTypeAnnotations != null && this.visibleTypeAnnotations.size() > 0) {
                throw new RuntimeException();
            }
            if (this.invisibleTypeAnnotations != null && this.invisibleTypeAnnotations.size() > 0) {
                throw new RuntimeException();
            }
            int n2 = (this.tryCatchBlocks == null) ? 0 : this.tryCatchBlocks.size();
            int i = 0;
            while (i < n2) {
                TryCatchBlockNode tryCatchBlockNode = (TryCatchBlockNode)this.tryCatchBlocks.get(i);
                if (tryCatchBlockNode.visibleTypeAnnotations != null && tryCatchBlockNode.visibleTypeAnnotations.size() > 0) {
                    throw new RuntimeException();
                }
                if (tryCatchBlockNode.invisibleTypeAnnotations != null && tryCatchBlockNode.invisibleTypeAnnotations.size() > 0) {
                    throw new RuntimeException();
                }
                ++i;
            }
            int j = 0;
            while (j < this.instructions.size()) {
                AbstractInsnNode value = this.instructions.get(j);
                if (value.visibleTypeAnnotations != null && value.visibleTypeAnnotations.size() > 0) {
                    throw new RuntimeException();
                }
                if (value.invisibleTypeAnnotations != null && value.invisibleTypeAnnotations.size() > 0) {
                    throw new RuntimeException();
                }
                if (value instanceof MethodInsnNode && ((MethodInsnNode)value).itf != (value.opcode == 185)) {
                    throw new RuntimeException();
                }
                ++j;
            }
            if (this.visibleLocalVariableAnnotations != null && this.visibleLocalVariableAnnotations.size() > 0) {
                throw new RuntimeException();
            }
            if (this.invisibleLocalVariableAnnotations != null && this.invisibleLocalVariableAnnotations.size() > 0) {
                throw new RuntimeException();
            }
        }
    }
    
    public void accept(ClassVisitor classVisitor) {
        String[] exceptions = new String[this.exceptions.size()];
        this.exceptions.toArray(exceptions);
        MethodVisitor visitMethod = classVisitor.visitMethod(this.access, this.name, this.desc, this.signature, exceptions);
        if (visitMethod != null) {
            this.accept(visitMethod);
        }
    }
    
    public void accept(MethodVisitor methodVisitor) {
        for (int n = (this.parameters == null) ? 0 : this.parameters.size(), i = 0; i < n; ++i) {
            ParameterNode parameterNode = (ParameterNode)this.parameters.get(i);
            methodVisitor.visitParameter(parameterNode.name, parameterNode.access);
        }
        if (this.annotationDefault != null) {
            AnnotationVisitor visitAnnotationDefault = methodVisitor.visitAnnotationDefault();
            AnnotationNode.accept(visitAnnotationDefault, null, this.annotationDefault);
            if (visitAnnotationDefault != null) {
                visitAnnotationDefault.visitEnd();
            }
        }
        for (int n2 = (this.visibleAnnotations == null) ? 0 : this.visibleAnnotations.size(), j = 0; j < n2; ++j) {
            AnnotationNode annotationNode = (AnnotationNode)this.visibleAnnotations.get(j);
            annotationNode.accept(methodVisitor.visitAnnotation(annotationNode.desc, true));
        }
        for (int n3 = (this.invisibleAnnotations == null) ? 0 : this.invisibleAnnotations.size(), k = 0; k < n3; ++k) {
            AnnotationNode annotationNode2 = (AnnotationNode)this.invisibleAnnotations.get(k);
            annotationNode2.accept(methodVisitor.visitAnnotation(annotationNode2.desc, false));
        }
        for (int n4 = (this.visibleTypeAnnotations == null) ? 0 : this.visibleTypeAnnotations.size(), l = 0; l < n4; ++l) {
            TypeAnnotationNode typeAnnotationNode = (TypeAnnotationNode)this.visibleTypeAnnotations.get(l);
            typeAnnotationNode.accept(methodVisitor.visitTypeAnnotation(typeAnnotationNode.typeRef, typeAnnotationNode.typePath, typeAnnotationNode.desc, true));
        }
        for (int n5 = (this.invisibleTypeAnnotations == null) ? 0 : this.invisibleTypeAnnotations.size(), n6 = 0; n6 < n5; ++n6) {
            TypeAnnotationNode typeAnnotationNode2 = (TypeAnnotationNode)this.invisibleTypeAnnotations.get(n6);
            typeAnnotationNode2.accept(methodVisitor.visitTypeAnnotation(typeAnnotationNode2.typeRef, typeAnnotationNode2.typePath, typeAnnotationNode2.desc, false));
        }
        for (int n7 = (this.visibleParameterAnnotations == null) ? 0 : this.visibleParameterAnnotations.length, parameter = 0; parameter < n7; ++parameter) {
            List<Object> list = this.visibleParameterAnnotations[parameter];
            if (list != null) {
                for (int n8 = 0; n8 < list.size(); ++n8) {
                    AnnotationNode annotationNode3 = (AnnotationNode)list.get(n8);
                    annotationNode3.accept(methodVisitor.visitParameterAnnotation(parameter, annotationNode3.desc, true));
                }
            }
        }
        for (int n9 = (this.invisibleParameterAnnotations == null) ? 0 : this.invisibleParameterAnnotations.length, parameter2 = 0; parameter2 < n9; ++parameter2) {
            List<Object> list2 = this.invisibleParameterAnnotations[parameter2];
            if (list2 != null) {
                for (int n10 = 0; n10 < list2.size(); ++n10) {
                    AnnotationNode annotationNode4 = (AnnotationNode)list2.get(n10);
                    annotationNode4.accept(methodVisitor.visitParameterAnnotation(parameter2, annotationNode4.desc, false));
                }
            }
        }
        if (this.visited) {
            this.instructions.resetLabels();
        }
        for (int n11 = (this.attrs == null) ? 0 : this.attrs.size(), n12 = 0; n12 < n11; ++n12) {
            methodVisitor.visitAttribute((Attribute)this.attrs.get(n12));
        }
        if (this.instructions.size() > 0) {
            methodVisitor.visitCode();
            for (int n13 = (this.tryCatchBlocks == null) ? 0 : this.tryCatchBlocks.size(), n14 = 0; n14 < n13; ++n14) {
                ((TryCatchBlockNode)this.tryCatchBlocks.get(n14)).updateIndex(n14);
                ((TryCatchBlockNode)this.tryCatchBlocks.get(n14)).accept(methodVisitor);
            }
            this.instructions.accept(methodVisitor);
            for (int n15 = (this.localVariables == null) ? 0 : this.localVariables.size(), n16 = 0; n16 < n15; ++n16) {
                ((LocalVariableNode)this.localVariables.get(n16)).accept(methodVisitor);
            }
            for (int n17 = (this.visibleLocalVariableAnnotations == null) ? 0 : this.visibleLocalVariableAnnotations.size(), n18 = 0; n18 < n17; ++n18) {
                ((LocalVariableAnnotationNode)this.visibleLocalVariableAnnotations.get(n18)).accept(methodVisitor, true);
            }
            for (int n19 = (this.invisibleLocalVariableAnnotations == null) ? 0 : this.invisibleLocalVariableAnnotations.size(), n20 = 0; n20 < n19; ++n20) {
                ((LocalVariableAnnotationNode)this.invisibleLocalVariableAnnotations.get(n20)).accept(methodVisitor, false);
            }
            methodVisitor.visitMaxs(this.maxStack, this.maxLocals);
            this.visited = true;
        }
        methodVisitor.visitEnd();
    }
}
